#!/usr/bin/python
# -*- coding: utf-8 -*-

# remote cv service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

import sys
import argparse
import logging

def main(params=None):
    if params is None:
        params = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Runs RemoteCV.')

    conn_group = parser.add_argument_group('Connection arguments')
    conn_group.add_argument('-b', '--bind', default='*', help='IP address to bind')
    conn_group.add_argument('-p', '--port', default=13337, type=int, help='Port to listen')
    conn_group.add_argument('-s', '--server-type', default='zmq', choices=['zmq', 'tornado'], help="Server type")

    other_group = parser.add_argument_group('Other arguments')
    other_group.add_argument('-l', '--level', default='debug', help='Logging level')

    arguments = parser.parse_args(params)
    logging.basicConfig(level=getattr(logging, arguments.level.upper()))

    server_module = __import__("%s_server" % arguments.server_type)
    server_module.run_server(arguments.bind, arguments.port)


if __name__ == "__main__":
    main()



import time
import traceback

import zmq

from remotecv.app import RemoteCvApp
from remotecv.utils import logger

def run_server(host, port):
    app = RemoteCvApp()
    context = zmq.Context()
    rep = context.socket(zmq.REP)
    bind_address = 'tcp://%s:%s' % (host, port)

    logger.debug('listening to connections at %s' % bind_address)

    rep.bind(bind_address)
    while True:
        result = ''
        msg = rep.recv()
        try:
            start_time = time.time()
            logger.debug('Accepting message')
            result = app.process_request(msg)
        except:
            logger.error(traceback.format_exc())

        rep.send(result)
        logger.debug('Sending response. Ellapsed: %d' % ((time.time() - start_time) * 1000))



from tornado.web import RequestHandler, Application
from tornado import httpserver, ioloop

from remotecv.app import RemoteCvApp
from remotecv.utils import logger

class RemoteCvProcessHandler(RequestHandler):
    remote_cv_app = RemoteCvApp()

    def post(self):
        data = self.request.body
        result = self.remote_cv_app.process_request(data)
        self.write(result)


class RemoteCvTornadoApp(Application):
    def __init__(self):
        handlers = [
            (r'/handle_image', RemoteCvProcessHandler)
        ]
        super(RemoteCvTornadoApp, self).__init__(handlers)


def run_server(host, port):
    application = RemoteCvTornadoApp()
    if host == '*':
        host = '0.0.0.0'

    logger.debug('listening to connections at http://%s:%d' % (host, port))

    server = httpserver.HTTPServer(application)
    server.bind(port, host)
    server.start(1)

    ioloop.IOLoop.instance().start()




import bson

from remotecv.image_processor import ImageProcessor

from remotecv.utils import logger

class RemoteCvApp:

    def __init__(self):
        self.processor = ImageProcessor()

    def process_request(self, msg):
        msg = bson.loads(msg)
        points = self.processor.detect(msg['type'], msg['size'], msg['mode'], msg['image'])
        return bson.dumps({ 'points': points })



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license: 
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

import cv

from remotecv.detectors import BaseDetector

class FeatureDetector(BaseDetector):

    def detect(self, width, height, mode, img_data):
        sz = (width, height)
        image = cv.CreateImageHeader(sz, cv.IPL_DEPTH_8U, 3)
        cv.SetData(image, img_data)

        gray_image = cv.CreateImage(sz, 8, 1);
        convert_mode = getattr(cv, 'CV_%s2GRAY' % mode)
        cv.CvtColor(image, gray_image, convert_mode)
        image = gray_image
        rows = sz[0]
        cols = sz[1]

        eig_image = cv.CreateMat(rows, cols, cv.CV_32FC1)
        temp_image = cv.CreateMat(rows, cols, cv.CV_32FC1)
        points = cv.GoodFeaturesToTrack(image, eig_image, temp_image, 20, 0.04, 1.0, useHarris = False)

        if points:
            return [[x, y, 1, 1] for x, y in points]

        return None
